/*
  MQTT.h - Library for GSM MQTT Client.
  Created by Nithin K. Kurian, Dhanish Vijayan, Elementz Engineers Guild Pvt. Ltd, July 2, 2016.
  Released into the public domain.
*/

#include "GSM_MQTT.h"
#include "Arduino.h"
#include <SoftwareSerial.h>
#include <avr/pgmspace.h>
extern uint8_t GSM_Response;

extern SoftwareSerial mySerial(10,11); 
extern String MQTT_HOST = "test.mosquitto.org";
extern String MQTT_PORT = "1883";

extern GSM_MQTT MQTT;
uint8_t GSM_Response = 0;
unsigned long previousMillis = 0;
unsigned long waitPeriod = 0;
//char inputString[UART_BUFFER_LENGTH];         // a string to hold incoming data
boolean stringComplete = false;  // whether the string is complete
void serialEvent();


GSM_MQTT::GSM_MQTT(unsigned long KeepAlive)
{
  _KeepAliveTimeOut = 20; //KeepAlive;
}

void GSM_MQTT::begin(int LogBauds, byte LogRX, byte LogTX, const char * MqttHost, const char * MqttPort)
{
  SoftwareSerial mySerial(LogRX,LogTX);
  MQTT_HOST = MqttHost;
  MQTT_PORT = MqttPort;
  mySerial.begin(LogBauds);
  Serial.begin(115200);
  Serial.write("AT\r\n");
  delay(1000);
  _tcpInit();
}
char GSM_MQTT::_sendAT(const char *command, unsigned long waitms)
{

  unsigned long PrevMillis = millis();
  strcpy(reply, "none");
  GSM_Response = 0;
 Serial.write(command);
  unsigned long currentMillis = millis();
  //  mySerial.println(PrevMillis);
  //  mySerial.println(currentMillis);
  while ( (GSM_Response == 0) && ((currentMillis - PrevMillis) < waitms) )
  {
    //    delay(1);
    serialEvent();
    currentMillis = millis();
  }
  return GSM_Response;
}
char GSM_MQTT::sendATreply(const char *command, const char *replystr, unsigned long waitms)
{
  strcpy(reply, replystr);
  unsigned long PrevMillis = millis();
  GSM_ReplyFlag = 0;
 Serial.write(command);
  unsigned long currentMillis = millis();

  //  mySerial.println(PrevMillis);
  //  mySerial.println(currentMillis);
  while ( (GSM_ReplyFlag == 0) && ((currentMillis - PrevMillis) < waitms) )
  {
    //    delay(1);
    serialEvent();
    currentMillis = millis();
  }
  return GSM_ReplyFlag;
}
void GSM_MQTT::_tcpInit(void)
{
  switch (modemStatus)
  {
    case 0:
      {
        delay(1000);
       Serial.print("+++");
        delay(500);
        if (_sendAT("AT\r\n", 5000) == 1)
        {
          modemStatus = 1;
        }
        else
        {
          modemStatus = 0;
          break;
        }
      }
    case 1:
      {
        if (_sendAT("ATE1\r\n", 2000) == 1)
        {
          modemStatus = 2;
        }
        else
        {
          modemStatus = 1;
          break;
        }
      }
    case 2:
      {
        if (sendATreply("AT+CREG?\r\n", "0,1", 5000) == 1)
        {
          _sendAT("AT+CIPMUX=0\r\n", 2000);
          _sendAT("AT+CIPMODE=1\r\n", 2000);
          if (sendATreply("AT+CGATT?\r\n", ": 1", 4000) != 1)
          {
            _sendAT("AT+CGATT=1\r\n", 2000);
          }
          modemStatus = 3;
          _tcpStatus = 2;
        }
        else
        {
          modemStatus = 2;
          break;
        }
      }
    case 3:
      {
        if (GSM_ReplyFlag != 7)
        {
          _tcpStatus = sendATreply("AT+CIPSTATUS\r\n", "STATE", 4000);
          if (_tcpStatusPrev == _tcpStatus)
          {
            tcpATerrorcount++;
            if (tcpATerrorcount >= 10)
            {
              tcpATerrorcount = 0;
              _tcpStatus = 7;
            }

          }
          else
          {
            _tcpStatusPrev = _tcpStatus;
            tcpATerrorcount = 0;
          }
        }
        _tcpStatusPrev = _tcpStatus;
        mySerial.print(_tcpStatus);
        switch (_tcpStatus)
        {
          case 2:
            {
              _sendAT("AT+CSTT=\"AIRTELGPRS.COM\"\r\n", 5000);
              break;
            }
          case 3:
            {
              _sendAT("AT+CIICR\r\n", 5000)  ;
              break;
            }
          case 4:
            {
              sendATreply("AT+CIFSR\r\n", ".", 4000) ;
              break;
            }
          case 5:
            {
             Serial.print("AT+CIPSTART=\"TCP\",\"");
             Serial.print(MQTT_HOST);
             Serial.print("\",\"");
             Serial.print(MQTT_PORT);
              if (_sendAT("\"\r\n", 5000) == 1)
              {
                unsigned long PrevMillis = millis();
                unsigned long currentMillis = millis();
                while ( (GSM_Response != 4) && ((currentMillis - PrevMillis) < 20000) )
                {
                  //    delay(1);
                  serialEvent();
                  currentMillis = millis();
                }
              }
              break;
            }
          case 6:
            {
              unsigned long PrevMillis = millis();
              unsigned long currentMillis = millis();
              while ( (GSM_Response != 4) && ((currentMillis - PrevMillis) < 20000) )
              {
                //    delay(1);
                serialEvent();
                currentMillis = millis();
              }
              break;
            }
          case 7:
            {
              sendATreply("AT+CIPSHUT\r\n", "OK", 4000) ;
              modemStatus = 0;
              _tcpStatus = 2;
              break;
            }
        }
      }
  }

}

void GSM_MQTT::GSM_MQTT::_ping(void)
{

  if (pingFlag == true)
  {
    unsigned long currentMillis = millis();
    if ((currentMillis - _PingPrevMillis ) >= _KeepAliveTimeOut * 1000)
    {
      // save the last time you blinked the LED
      _PingPrevMillis = currentMillis;
      if(pingSent){
        mySerial.println("Ping failed");
        disconnect();
        TCP_Flag = false;
        MQTT_Flag = false;
        pingFlag = false;
        sendATreply("AT+CIPSHUT\r\n", ".", 4000);
        modemStatus = 0;
        pingSent = false;
        return;
      }
     Serial.print(char(PINGREQ * 16));
      _sendLength(0);
      pingSent = true;
      mySerial.println("pingSent = true");
    }
  }
}
void GSM_MQTT::_sendUTFString(const char *string)
{
  int localLength = strlen(string);
 Serial.print(char(localLength / 256));
 Serial.print(char(localLength % 256));
 Serial.print(string);
}
void GSM_MQTT::_sendLength(int len)
{
  bool  length_flag = false;
  while (length_flag == false)
  {
    if ((len / 128) > 0)
    {
     Serial.print(char(len % 128 + 128));
      len /= 128;
    }
    else
    {
      length_flag = true;
     Serial.print(char(len));
    }
  }
}
void GSM_MQTT::connect(const char *ClientIdentifier, char UserNameFlag, char PasswordFlag, const char *UserName, const char *Password, char CleanSession, char WillFlag, char WillQoS, char WillRetain, const char *WillTopic, const char *WillMessage)
{
  ConnectionAcknowledgement = NO_ACKNOWLEDGEMENT ;
 Serial.print(char(CONNECT * 16 ));
  char ProtocolName[7] = "MQIsdp";
  int localLength = (2 + strlen(ProtocolName)) + 1 + 3 + (2 + strlen(ClientIdentifier));
  if (WillFlag != 0)
  {
    localLength = localLength + 2 + strlen(WillTopic) + 2 + strlen(WillMessage);
  }
  if (UserNameFlag != 0)
  {
    localLength = localLength + 2 + strlen(UserName);

    if (PasswordFlag != 0)
    {
      localLength = localLength + 2 + strlen(Password);
    }
  }
  _sendLength(localLength);
  _sendUTFString(ProtocolName);
 Serial.print(char(_ProtocolVersion));
 Serial.print(char(UserNameFlag * User_Name_Flag_Mask + PasswordFlag * Password_Flag_Mask + WillRetain * Will_Retain_Mask + WillQoS * Will_QoS_Scale + WillFlag * Will_Flag_Mask + CleanSession * Clean_Session_Mask));
 Serial.print(char(_KeepAliveTimeOut / 256));
 Serial.print(char(_KeepAliveTimeOut % 256));
  _sendUTFString(ClientIdentifier);
  if (WillFlag != 0)
  {
    _sendUTFString(WillTopic);
    _sendUTFString(WillMessage);
  }
  if (UserNameFlag != 0)
  {
    _sendUTFString(UserName);
    if (PasswordFlag != 0)
    {
      _sendUTFString(Password);
    }
  }
}
void GSM_MQTT::publish(char DUP, char Qos, char RETAIN, unsigned int MessageID, const char *Topic, const char *Message)
{
 Serial.print(char(PUBLISH * 16 + DUP * DUP_Mask + Qos * QoS_Scale + RETAIN));
  int localLength = (2 + strlen(Topic));
  if (Qos > 0)
  {
    localLength += 2;
  }
  localLength += strlen(Message);
  _sendLength(localLength);
  _sendUTFString(Topic);
  if (Qos > 0)
  {
   Serial.print(char(MessageID / 256));
   Serial.print(char(MessageID % 256));
  }
 Serial.print(Message);
}
void GSM_MQTT::publishACK(unsigned int MessageID)
{
 Serial.print(char(PUBACK * 16));
  _sendLength(2);
 Serial.print(char(MessageID / 256));
 Serial.print(char(MessageID % 256));
}
void GSM_MQTT::publishREC(unsigned int MessageID)
{
 Serial.print(char(PUBREC * 16));
  _sendLength(2);
 Serial.print(char(MessageID / 256));
 Serial.print(char(MessageID % 256));
}
void GSM_MQTT::publishREL(char DUP, unsigned int MessageID)
{
 Serial.print(char(PUBREL * 16 + DUP * DUP_Mask + 1 * QoS_Scale));
  _sendLength(2);
 Serial.print(char(MessageID / 256));
 Serial.print(char(MessageID % 256));
}

void GSM_MQTT::publishCOMP(unsigned int MessageID)
{
 Serial.print(char(PUBCOMP * 16));
  _sendLength(2);
 Serial.print(char(MessageID / 256));
 Serial.print(char(MessageID % 256));
}
void GSM_MQTT::subscribe(char DUP, unsigned int MessageID,const char *SubTopic, char SubQoS)
{
 Serial.print(char(SUBSCRIBE * 16 + DUP * DUP_Mask + 1 * QoS_Scale));
  int localLength = 2 + (2 + strlen(SubTopic)) + 1;
  _sendLength(localLength);
 Serial.print(char(MessageID / 256));
 Serial.print(char(MessageID % 256));
  _sendUTFString(SubTopic);
 Serial.print(SubQoS);

}
void GSM_MQTT::unsubscribe(char DUP, unsigned int MessageID, const char *SubTopic)
{
 Serial.print(char(UNSUBSCRIBE * 16 + DUP * DUP_Mask + 1 * QoS_Scale));
  int localLength = (2 + strlen(SubTopic)) + 2;
  _sendLength(localLength);

 Serial.print(char(MessageID / 256));
 Serial.print(char(MessageID % 256));

  _sendUTFString(SubTopic);
}
void GSM_MQTT::disconnect(void)
{
 Serial.print(char(DISCONNECT * 16));
  _sendLength(0);
  pingFlag = false;
}
//Messages
const char CONNECTMessage[] PROGMEM  = {"Client request to connect to Server\r\n"};
const char CONNACKMessage[] PROGMEM  = {"Connect Acknowledgment\r\n"};
const char PUBLISHMessage[] PROGMEM  = {"Publish message\r\n"};
const char PUBACKMessage[] PROGMEM  = {"Publish Acknowledgment\r\n"};
const char PUBRECMessage[] PROGMEM  = {"Publish Received (assured delivery part 1)\r\n"};
const char PUBRELMessage[] PROGMEM  = {"Publish Release (assured delivery part 2)\r\n"};
const char PUBCOMPMessage[] PROGMEM  = {"Publish Complete (assured delivery part 3)\r\n"};
const char SUBSCRIBEMessage[] PROGMEM  = {"Client Subscribe request\r\n"};
const char SUBACKMessage[] PROGMEM  = {"Subscribe Acknowledgment\r\n"};
const char UNSUBSCRIBEMessage[] PROGMEM  = {"Client Unsubscribe request\r\n"};
const char UNSUBACKMessage[] PROGMEM  = {"Unsubscribe Acknowledgment\r\n"};
const char PINGREQMessage[] PROGMEM  = {"PING Request\r\n"};
const char PINGRESPMessage[] PROGMEM  = {"PING Response\r\n"};
const char DISCONNECTMessage[] PROGMEM  = {"Client is Disconnecting\r\n"};

void GSM_MQTT::printMessageType(uint8_t Message)
{
  switch (Message)
  {
    case CONNECT:
      {
        int k, len = strlen_P(CONNECTMessage);
        char myChar;
        for (k = 0; k < len; k++)
        {
          myChar =  pgm_read_byte_near(CONNECTMessage + k);
          mySerial.print(myChar);
        }
        break;
      }
    case CONNACK:
      {
        int k, len = strlen_P(CONNACKMessage);
        char myChar;
        for (k = 0; k < len; k++)
        {
          myChar =  pgm_read_byte_near(CONNACKMessage + k);
          mySerial.print(myChar);
        }
        break;
      }
    case PUBLISH:
      {
        int k, len = strlen_P(PUBLISHMessage);
        char myChar;
        for (k = 0; k < len; k++)
        {
          myChar =  pgm_read_byte_near(PUBLISHMessage + k);
          mySerial.print(myChar);
        }
        break;
      }
    case PUBACK:
      {
        int k, len = strlen_P(PUBACKMessage);
        char myChar;
        for (k = 0; k < len; k++)
        {
          myChar =  pgm_read_byte_near(PUBACKMessage + k);
          mySerial.print(myChar);
        }
        break;
      }
    case  PUBREC:
      {
        int k, len = strlen_P(PUBRECMessage);
        char myChar;
        for (k = 0; k < len; k++)
        {
          myChar =  pgm_read_byte_near(PUBRECMessage + k);
          mySerial.print(myChar);
        }
        break;
      }
    case PUBREL:
      {
        int k, len = strlen_P(PUBRELMessage);
        char myChar;
        for (k = 0; k < len; k++)
        {
          myChar =  pgm_read_byte_near(PUBRELMessage + k);
          mySerial.print(myChar);
        }
        break;
      }
    case PUBCOMP:
      {
        int k, len = strlen_P(PUBCOMPMessage );
        char myChar;
        for (k = 0; k < len; k++)
        {
          myChar =  pgm_read_byte_near(PUBCOMPMessage  + k);
          mySerial.print(myChar);
        }
        break;
      }
    case SUBSCRIBE:
      {
        int k, len = strlen_P(SUBSCRIBEMessage );
        char myChar;
        for (k = 0; k < len; k++)
        {
          myChar =  pgm_read_byte_near(SUBSCRIBEMessage  + k);
          mySerial.print(myChar);
        }
        break;
      }
    case SUBACK:
      {
        int k, len = strlen_P(SUBACKMessage );
        char myChar;
        for (k = 0; k < len; k++)
        {
          myChar =  pgm_read_byte_near(SUBACKMessage  + k);
          mySerial.print(myChar);
        }
        break;
      }
    case UNSUBSCRIBE:
      {
        int k, len = strlen_P(UNSUBSCRIBEMessage );
        char myChar;
        for (k = 0; k < len; k++)
        {
          myChar =  pgm_read_byte_near(UNSUBSCRIBEMessage  + k);
          mySerial.print(myChar);
        }
        break;
      }
    case UNSUBACK:
      {
        int k, len = strlen_P(UNSUBACKMessage );
        char myChar;
        for (k = 0; k < len; k++)
        {
          myChar =  pgm_read_byte_near(UNSUBACKMessage  + k);
          mySerial.print(myChar);
        }
        break;
      }
    case PINGREQ:
      {
        int k, len = strlen_P(PINGREQMessage);
        char myChar;
        for (k = 0; k < len; k++)
        {
          myChar =  pgm_read_byte_near(PINGREQMessage + k);
          mySerial.print(myChar);
        }
        break;
      }
    case PINGRESP:
      {
        pingSent = false;
        mySerial.println("pingSent = false");
        int k, len = strlen_P(PINGRESPMessage);
        char myChar;
        for (k = 0; k < len; k++)
        {
          myChar =  pgm_read_byte_near(PINGRESPMessage + k);
          mySerial.print(myChar);
        }
        break;
      }
    case DISCONNECT:
      {
        int k, len = strlen_P(DISCONNECTMessage);
        char myChar;
        for (k = 0; k < len; k++)
        {
          myChar =  pgm_read_byte_near(DISCONNECTMessage + k);
          mySerial.print(myChar);
        }
        break;
      }
  }
}

//Connect Ack
const char ConnectAck0[] PROGMEM  = {"Connection Accepted\r\n"};
const char ConnectAck1[] PROGMEM  = {"Connection Refused: unacceptable protocol version\r\n"};
const char ConnectAck2[] PROGMEM  = {"Connection Refused: identifier rejected\r\n"};
const char ConnectAck3[] PROGMEM  = {"Connection Refused: server unavailable\r\n"};
const char ConnectAck4[] PROGMEM  = {"Connection Refused: bad user name or password\r\n"};
const char ConnectAck5[] PROGMEM  = {"Connection Refused: not authorized\r\n"};
void GSM_MQTT::printConnectAck(uint8_t Ack)
{
  switch (Ack)
  {
    case 0:
      {
        int k, len = strlen_P(ConnectAck0);
        char myChar;
        for (k = 0; k < len; k++)
        {
          myChar =  pgm_read_byte_near(ConnectAck0 + k);
          mySerial.print(myChar);
        }
        break;
      }
    case 1:
      {
        int k, len = strlen_P(ConnectAck1);
        char myChar;
        for (k = 0; k < len; k++)
        {
          myChar =  pgm_read_byte_near(ConnectAck1 + k);
          mySerial.print(myChar);
        }
        break;
      }
    case 2:
      {
        int k, len = strlen_P(ConnectAck2);
        char myChar;
        for (k = 0; k < len; k++)
        {
          myChar =  pgm_read_byte_near(ConnectAck2 + k);
          mySerial.print(myChar);
        }
        break;
      }
    case 3:
      {
        int k, len = strlen_P(ConnectAck3);
        char myChar;
        for (k = 0; k < len; k++)
        {
          myChar =  pgm_read_byte_near(ConnectAck3 + k);
          mySerial.print(myChar);
        }
        break;
      }
    case 4:
      {
        int k, len = strlen_P(ConnectAck4);
        char myChar;
        for (k = 0; k < len; k++)
        {
          myChar =  pgm_read_byte_near(ConnectAck4 + k);
          mySerial.print(myChar);
        }
        break;
      }
    case 5:
      {
        int k, len = strlen_P(ConnectAck5);
        char myChar;
        for (k = 0; k < len; k++)
        {
          myChar =  pgm_read_byte_near(ConnectAck5 + k);
          mySerial.print(myChar);
        }
        break;
      }
  }
}
unsigned int GSM_MQTT::_generateMessageID(void)
{
  if (_LastMessaseID < 65535)
  {
    return ++_LastMessaseID;
  }
  else
  {
    _LastMessaseID = 0;
    return _LastMessaseID;
  }
}
void GSM_MQTT::processing(void)
{
  unsigned long t = millis(); 
  if (TCP_Flag == false)
  {
    MQTT_Flag = false;
//    _tcpInit();
  }
//  _ping(); 
// modification   
  if (t - waitPeriod > 1000L) 
  {
    waitPeriod = t;
    serialEvent(); //call serialEvent every second
    if (TCP_Flag == false)
    {
      _tcpInit();
    }
    _ping();    
  }  
  
}
bool GSM_MQTT::available(void)
{
  return MQTT_Flag;
}

bool GSM_MQTT::OnConnect(void)
{
  if (MQTT_Start == true)
  { 
  MQTT_Start = false; 
  return true; }
  else
  {return false; }
}
bool GSM_MQTT::AutoConnect(void)
{
  if (MQTT_Stop == true)
  { 
  MQTT_Stop = false; 
  return true; }
  else
  {return false; }
}
bool GSM_MQTT::OnMessage(void)
{
  if (MQTT_NewMessage == true)
  { 
  MQTT_NewMessage = false; 
  return true; }
  else
  {return false; }
}

//void GSM_MQTT::ReadMessage(char * lTopic, int lTopicLength, byte * lMessage, int lMessageLength)
//{
//  for(int i=0; i<TopicLength; i++) // Store the message in the buffer
//    { lTopic[i] = Topic[i];  }
//  lTopicLength = TopicLength;  
//  for(int i=0; i<MessageLength; i++) // Store the message in the buffer
//    { lTopic[i] = Message[i]; }                                        
//  lMessageLength = MessageLength;
//}

void GSM_MQTT::Log(const char * Mess)
{
   mySerial.println(Mess);

}


//----------------------------------------------------------------------------
void GSM_MQTT::serialEvent()
{
  while (Serial.available())
  {
    char inChar = (char)Serial.read();
    if (TCP_Flag == false)
    { 
      if (index < 200)
      {
        inputString[index++] = inChar;
      }
      if (inChar == '\n')
      {
        inputString[index] = 0;
        stringComplete = true;
        mySerial.print(inputString);
        if (strstr(inputString, reply) != NULL)
        {
          GSM_ReplyFlag = 1;
          if (strstr(inputString, " INITIAL") != 0)
          {
            GSM_ReplyFlag = 2; //
          }
          else if (strstr(inputString, " START") != 0)
          {
            GSM_ReplyFlag = 3; //
          }
          else if (strstr(inputString, "IP CONFIG") != 0)
          {
   //         _delay_us(10);
            GSM_ReplyFlag = 4;
          }
          else if (strstr(inputString, " GPRSACT") != 0)
          {
            GSM_ReplyFlag = 4; //
          }
          else if ((strstr(inputString, " STATUS") != 0) || (strstr(inputString, "TCP CLOSED") != 0))
          {
            GSM_ReplyFlag = 5; //
          }
          else if (strstr(inputString, " TCP CONNECTING") != 0)
          {
            GSM_ReplyFlag = 6; //
          }
          else if ((strstr(inputString, " CONNECT OK") != 0) || (strstr(inputString, "CONNECT FAIL") != NULL) || (strstr(inputString, "PDP DEACT") != 0))
          {
            GSM_ReplyFlag = 7;
          }
        }
        else if (strstr(inputString, "OK") != NULL)
        {
          GSM_Response = 1;
        }
        else if (strstr(inputString, "ERROR") != NULL)
        {
          GSM_Response = 2;
        }
        else if (strstr(inputString, ".") != NULL)
        {
          GSM_Response = 3;
        }
        else if (strstr(inputString, "CONNECT FAIL") != NULL)
        {
          GSM_Response = 5;
        }
        else if (strstr(inputString, "CONNECT") != NULL)
        {
          GSM_Response = 4;
          TCP_Flag = true;
          mySerial.println("TCP_Flag = True");
 //         AutoConnect();
          pingFlag = true;
          tcpATerrorcount = 0;
        }
        else if (strstr(inputString, "CLOSED") != NULL)
        {
          GSM_Response = 4;
          TCP_Flag = false;
          MQTT_Flag = false;
        }
        index = 0;
        inputString[0] = 0;
      }
    }   
    else
    {
      uint8_t ReceivedMessageType = (inChar / 16) & 0x0F;
      uint8_t DUP = (inChar & DUP_Mask) / DUP_Mask;
      uint8_t QoS = (inChar & QoS_Mask) / QoS_Scale;
      uint8_t RETAIN = (inChar & RETAIN_Mask);
      if ((ReceivedMessageType >= CONNECT) && (ReceivedMessageType <= DISCONNECT))
      {
        bool NextLengthByte = true;
        length = 0;
        lengthLocal = 0;
        uint32_t multiplier=1;
        delay(2);
        char Cchar = inChar;
        while ( (NextLengthByte == true) && (TCP_Flag == true))
        {
          if (Serial.available())
          {
            inChar = (char)Serial.read();
            mySerial.println(inChar, DEC);
            if ((((Cchar & 0xFF) == 'C') && ((inChar & 0xFF) == 'L') && (length == 0)) || (((Cchar & 0xFF) == '+') && ((inChar & 0xFF) == 'P') && (length == 0)))
            {
              index = 0;
              inputString[index++] = Cchar;
              inputString[index++] = inChar;
              TCP_Flag = false;
              MQTT_Flag = false;
              pingFlag = false;
              mySerial.println("Disconnecting");
            }
            else
            {
              if ((inChar & 128) == 128)
              {
                length += (inChar & 127) *  multiplier;
                multiplier *= 128;
                mySerial.println("More");
              }
              else
              {
                NextLengthByte = false;
                length += (inChar & 127) *  multiplier;
                multiplier *= 128;
              }
            }
          }
        }
        lengthLocal = length;
        mySerial.println(length);
        if (TCP_Flag == true)
        {
          printMessageType(ReceivedMessageType);
          index = 0L;
          uint32_t a = 0;
          while ((length-- > 0) && (Serial.available()))
          {
            inputString[uint32_t(index++)] = (char)Serial.read();

            delay(1);

          }
          mySerial.println(" ");
          if (ReceivedMessageType == CONNACK)
          {
            ConnectionAcknowledgement = inputString[0] * 256 + inputString[1];
            if (ConnectionAcknowledgement == 0)
            {
              MQTT_Flag = true;
 //             OnConnect();

            }

            printConnectAck(ConnectionAcknowledgement);
            // OnConnect();
          }
          else if (ReceivedMessageType == PUBLISH)
          {
            uint32_t TopicLength = (inputString[0]) * 256 + (inputString[1]);
            mySerial.print("Topic : '");
            PublishIndex = 0;
            for (uint32_t iter = 2; iter < TopicLength + 2; iter++)
            {
              mySerial.print(inputString[iter]);
              Topic[PublishIndex++] = inputString[iter];
            }
            Topic[PublishIndex] = 0;
            mySerial.print("' Message :'");
            TopicLength = PublishIndex;

            PublishIndex = 0;
            uint32_t MessageSTART = TopicLength + 2UL;
            int MessageID = 0;
            if (QoS != 0)
            {
              MessageSTART += 2;
              MessageID = inputString[TopicLength + 2UL] * 256 + inputString[TopicLength + 3UL];
            }
            for (uint32_t iter = (MessageSTART); iter < (lengthLocal); iter++)
            {
              mySerial.print(inputString[iter]);
              Message[PublishIndex++] = inputString[iter];
            }
            Message[PublishIndex] = 0;
            mySerial.println("'");
            MessageLength = PublishIndex;
            if (QoS == 1)
            {
              publishACK(MessageID);
            }
            else if (QoS == 2)
            {
              publishREC(MessageID);
            }
//            OnMessage(Topic, TopicLength, Message, MessageLength);
            MessageFlag = true;
          }
          else if (ReceivedMessageType == PUBREC)
          {
            mySerial.print("Message ID :");
            publishREL(0, inputString[0] * 256 + inputString[1]) ;
            mySerial.println(inputString[0] * 256 + inputString[1]) ;

          }
          else if (ReceivedMessageType == PUBREL)
          {
            mySerial.print("Message ID :");
            publishCOMP(inputString[0] * 256 + inputString[1]) ;
            mySerial.println(inputString[0] * 256 + inputString[1]) ;

          }
          else if ((ReceivedMessageType == PUBACK) || (ReceivedMessageType == PUBCOMP) || (ReceivedMessageType == SUBACK) || (ReceivedMessageType == UNSUBACK))
          {
            mySerial.print("Message ID :");
            mySerial.println(inputString[0] * 256 + inputString[1]) ;
          }
          else if (ReceivedMessageType == PINGREQ)
          {
            TCP_Flag = false;
    //        MQTT_flag = false;
            pingFlag = false;
            mySerial.println("Disconnecting");
            sendATreply("AT+CIPSHUT\r\n", ".", 4000) ;
            modemStatus = 0;
          }
        }
      }
      else if ((inChar = 13) || (inChar == 10))
      {
      }
      else
      {
        mySerial.print("Received :Unknown Message Type :");
        mySerial.println(inChar);
      }
    }
}
 }
