
#include "B4RDefines.h"

namespace B4R
{

    void B4RGSM_MQTT::Initialize(ulong KeepAlive, int LogBauds,byte LogRX,byte LogTX,B4RString* MqttHost,B4RString* MqttPort, SubVoid1Void AutoConnectSub, SubVoid2Void OnConnectSub, SubVoid1StringBArray OnMessageSub)
    {
       rcs = new (backend) GSM_MQTT(KeepAlive);
       this->AutoConnectSub = AutoConnectSub; 
       this->OnConnectSub = OnConnectSub; 
       this->OnMessageSub = OnMessageSub; 
       FunctionUnion fu; 
       fu.PollerFunction = looper;
       pnode.functionUnion = fu;
       pnode.tag = this; 
       if (pnode.next == NULL) 
          pollers.add(&pnode); 
       rcs->begin(LogBauds, LogRX, LogTX, MqttHost->data, MqttPort->data);
    };

    void B4RGSM_MQTT::looper(void* b) 
    {
       B4RGSM_MQTT* me = (B4RGSM_MQTT*)b;
       me->rcs->processing();
       if (me->rcs->AutoConnect()) { 
         me->AutoConnectSub();
         };
       if (me->rcs->OnConnect()) { 
         me->OnConnectSub();
         };
       if (me->rcs->OnMessage()) { 
         const UInt cp = B4R::StackMemory::cp;
         B4RString* strTopic = CreateStackMemoryObject(B4RString);
         strTopic->data = me->rcs->Topic;
         ArrayByte* arrMessage = CreateStackMemoryObject(ArrayByte);
         arrMessage->data = me->rcs->Message;
         arrMessage->length = me->rcs->MessageLength;
         me->OnMessageSub(strTopic, arrMessage);
         B4R::StackMemory::cp = cp;
         };
    };


    void B4RGSM_MQTT::connect(B4RString* ClientIdentifier, byte UserNameFlag, byte PassuintFlag, B4RString* UserName, B4RString* Passuint, byte CleanSession, byte WillFlag, byte WillQoS, byte WillRetain, B4RString* WillTopic, B4RString* WillMessage)
    {
       rcs->connect(ClientIdentifier->data, UserNameFlag, PassuintFlag, UserName->data, Passuint->data, CleanSession, WillFlag, WillQoS, WillRetain, WillTopic->data, WillMessage->data);
    };

    void B4RGSM_MQTT::publish(byte DUP, byte Qos, byte RETAIN, uint MessageID, B4RString* Topic, B4RString* Message)
    {
       rcs->publish(DUP, Qos, RETAIN, MessageID, Topic->data, Message->data);
    };

    void B4RGSM_MQTT::subscribe(byte DUP, uint MessageID, B4RString* SubTopic, byte SubQoS)
    {
       rcs->subscribe(DUP, MessageID, SubTopic->data, SubQoS);
    };

    void B4RGSM_MQTT::unsubscribe(byte DUP, uint MessageID, B4RString* SubTopic)
    {
       rcs->unsubscribe(DUP, MessageID, SubTopic->data);
    };

    void B4RGSM_MQTT::disconnect()
    {
       rcs->disconnect();
    };

    bool B4RGSM_MQTT::available()
    {
       return rcs->available();
    };

    void B4RGSM_MQTT::Log(B4RString* Mess)
    {
       rcs->Log(Mess->data);
    };

    void B4RGSM_MQTT::publishACK(uint MessageID)
    {
       rcs->publishACK(MessageID);
    };

    void B4RGSM_MQTT::publishREC(uint MessageID)
    {
       rcs->publishREC(MessageID);
    };

    void B4RGSM_MQTT::publishREL(byte DUP, uint MessageID)
    {
       rcs->publishREL(DUP, MessageID);
    };

    void B4RGSM_MQTT::publishCOMP(uint MessageID)
    {
       rcs->publishCOMP(MessageID);
    };

    void B4RGSM_MQTT::printMessageType(byte Message)
    {
       rcs->printMessageType(Message);
    };

    void B4RGSM_MQTT::printConnectAck(byte Ack)
    {
       rcs->printConnectAck(Ack);
    };

    byte B4RGSM_MQTT::sendATreply(B4RString* command, B4RString* replystr, ulong waitms)
    {
       return rcs->sendATreply(command->data, replystr->data, waitms);
    };


}


